@@ -3,7 +3,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -26,11 +26,13 @@ import unittest
 
 def test_suite():
     names = [
+        "bundle",
         "check_ignore",
         "client",
         "commit_graph",
         "dumb",
         "index",
+        "lfs",
         "pack",
         "patch",
         "porcelain",
