@@ -3,7 +3,7 @@
 #
 # SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -91,7 +91,7 @@ Subject: [PATCH 1/2] Remove executable bit from prey.ico (triggers a warning).
         self.assertEqual(b"Jelmer Vernooij <jelmer@samba.org>", c.committer)
         self.assertEqual(b"Jelmer Vernooij <jelmer@samba.org>", c.author)
         self.assertEqual(
-            b"Remove executable bit from prey.ico " b"(triggers a warning).\n",
+            b"Remove executable bit from prey.ico (triggers a warning).\n",
             c.message,
         )
         self.assertEqual(
@@ -123,7 +123,7 @@ Subject: [PATCH 1/2] Remove executable bit from prey.ico (triggers a warning).
         self.assertEqual(b"Jelmer Vernooij <jelmer@samba.org>", c.committer)
         self.assertEqual(b"Jelmer Vernooij <jelmer@samba.org>", c.author)
         self.assertEqual(
-            b"Remove executable bit from prey.ico " b"(triggers a warning).\n",
+            b"Remove executable bit from prey.ico (triggers a warning).\n",
             c.message,
         )
         self.assertEqual(
@@ -220,7 +220,7 @@ From: Jelmer Vernooij <jelmer@debian.org>
 
     def test_extract_mercurial(self) -> NoReturn:
         raise SkipTest(
-            "git_am_patch_split doesn't handle Mercurial patches " "properly yet"
+            "git_am_patch_split doesn't handle Mercurial patches properly yet"
         )
         expected_diff = """\
 diff --git a/dulwich/tests/test_patch.py b/dulwich/tests/test_patch.py
